@@ -34,7 +34,7 @@
 
 #include <ctype.h>    // for isspace()
 #include <stdlib.h>   // for getenv()
-#include <stdio.h>    // for snprintf(), sscanf()
+#include <stdio.h>    // IWYU pragma: keep
 #include <string.h>   // for memmove(), memchr(), etc.
 #include <fcntl.h>    // for open()
 #include <errno.h>    // for errno
@@ -71,7 +71,21 @@
 #include <sys/sysmacros.h>
 #endif
 
+#if __FreeBSD__ && !defined TCMALLOC_WANT_DL_ITERATE_PHDR
+#define TCMALLOC_WANT_DL_ITERATE_PHDR
+#endif
+
+#if defined TCMALLOC_WANT_DL_ITERATE_PHDR
+#include <link.h>
+#endif
+
+#include <string>
+
+#include "base/function_ref.h"
 #include "base/logging.h"
+#include "base/sysinfo.h"
+
+#include "base/for_each_line.h"
 
 #ifdef PLATFORM_WINDOWS
 #ifdef MODULEENTRY32
@@ -99,17 +113,37 @@ namespace tcmalloc {
 
 namespace {
 
+template <typename Body>
+bool DoForEachLine(const char* path, const Body& body) {
+  int fd;
+  NO_INTR(fd = open(path, O_RDONLY));
+  if (fd < 0) {
+    return false;
+  }
+
+  auto reader = [fd] (void* buf, int sz) -> int {
+    int nread;
+    NO_INTR(nread = read(fd, buf, sz));
+    return nread;
+  };
+
+  bool ok = ForEachLine<1024>(reader, body);
+
+  (void)close(fd);
+  return ok;
+}
+
 // Finds |c| in |text|, and assign '\0' at the found position.
 // The original character at the modified position should be |c|.
 // A pointer to the modified position is stored in |endptr|.
-// |endptr| should not be NULL.
+// |endptr| should not be nullptr.
 bool ExtractUntilChar(char *text, int c, char **endptr) {
-  CHECK_NE(text, NULL);
-  CHECK_NE(endptr, NULL);
+  CHECK_NE(text, nullptr);
+  CHECK_NE(endptr, nullptr);
   char *found;
   found = strchr(text, c);
-  if (found == NULL) {
-    *endptr = NULL;
+  if (found == nullptr) {
+    *endptr = nullptr;
     return false;
   }
 
@@ -128,40 +162,18 @@ void SkipWhileWhitespace(char **text_pointer, int c) {
   }
 }
 
-template<class T>
-T StringToInteger(char *text, char **endptr, int base) {
-  assert(false);
-  return T();
-}
-
-template<>
-inline int StringToInteger<int>(char *text, char **endptr, int base) {
-  return strtol(text, endptr, base);
-}
-
-template<>
-inline int64_t StringToInteger<int64_t>(char *text, char **endptr, int base) {
-  return strtoll(text, endptr, base);
-}
-
-template<>
-inline uint64_t StringToInteger<uint64_t>(char *text, char **endptr, int base) {
-  return strtoull(text, endptr, base);
-}
-
-template<typename T>
-T StringToIntegerUntilChar(
+uint64_t StringToIntegerUntilChar(
     char *text, int base, int c, char **endptr_result) {
-  CHECK_NE(endptr_result, NULL);
-  *endptr_result = NULL;
+  CHECK_NE(endptr_result, nullptr);
+  *endptr_result = nullptr;
 
   char *endptr_extract;
   if (!ExtractUntilChar(text, c, &endptr_extract))
     return 0;
 
-  T result;
+  uint64_t result;
   char *endptr_strto;
-  result = StringToInteger<T>(text, &endptr_strto, base);
+  result = strtoull(text, &endptr_strto, base);
   *endptr_extract = c;
 
   if (endptr_extract != endptr_strto)
@@ -177,7 +189,7 @@ char *CopyStringUntilChar(
     char *text, unsigned out_len, int c, char *out) {
   char *endptr;
   if (!ExtractUntilChar(text, c, &endptr))
-    return NULL;
+    return nullptr;
 
   strncpy(out, text, out_len);
   out[out_len-1] = '\0';
@@ -187,27 +199,26 @@ char *CopyStringUntilChar(
   return endptr;
 }
 
-template<typename T>
 bool StringToIntegerUntilCharWithCheck(
-    T *outptr, char *text, int base, int c, char **endptr) {
-  *outptr = StringToIntegerUntilChar<T>(*endptr, base, c, endptr);
-  if (*endptr == NULL || **endptr == '\0') return false;
+    uint64_t *outptr, char *text, int base, int c, char **endptr) {
+  *outptr = StringToIntegerUntilChar(*endptr, base, c, endptr);
+  if (*endptr == nullptr || **endptr == '\0') return false;
   ++(*endptr);
   return true;
 }
 
+#if defined(__linux__) || defined(__NetBSD__)
 bool ParseProcMapsLine(char *text, uint64_t *start, uint64_t *end,
                        char *flags, uint64_t *offset,
-                       int64_t *inode,
+                       uint64_t *inode,
                        unsigned *filename_offset) {
-#if defined(__linux__) || defined(__NetBSD__)
   /*
    * It's similar to:
    * sscanf(text, "%"SCNx64"-%"SCNx64" %4s %"SCNx64" %x:%x %"SCNd64" %n",
    *        start, end, flags, offset, major, minor, inode, filename_offset)
    */
   char *endptr = text;
-  if (endptr == NULL || *endptr == '\0')  return false;
+  if (endptr == nullptr || *endptr == '\0')  return false;
 
   if (!StringToIntegerUntilCharWithCheck(start, endptr, 16, '-', &endptr))
     return false;
@@ -216,13 +227,13 @@ bool ParseProcMapsLine(char *text, uint64_t *start, uint64_t *end,
     return false;
 
   endptr = CopyStringUntilChar(endptr, 5, ' ', flags);
-  if (endptr == NULL || *endptr == '\0')  return false;
+  if (endptr == nullptr || *endptr == '\0')  return false;
   ++endptr;
 
   if (!StringToIntegerUntilCharWithCheck(offset, endptr, 16, ' ', &endptr))
     return false;
 
-  int64_t dummy;
+  uint64_t dummy;
   if (!StringToIntegerUntilCharWithCheck(&dummy, endptr, 16, ':', &endptr))
     return false;
 
@@ -234,99 +245,10 @@ bool ParseProcMapsLine(char *text, uint64_t *start, uint64_t *end,
 
   *filename_offset = (endptr - text);
   return true;
-#else
-  return false;
-#endif
 }
 
-template <typename Body>
-bool ForEachLine(const char* path, const Body& body) {
-#ifdef __FreeBSD__
-  // FreeBSD requires us to read all of the maps file at once, so
-  // we have to make a buffer that's "always" big enough
-  // TODO(alk): thats not good enough. We can do better.
-  static constexpr size_t kBufSize = 102400;
-#else   // a one-line buffer is good enough
-  static constexpr size_t kBufSize = PATH_MAX + 1024;
-#endif
-  char buf[kBufSize];
-  char* const buf_end = buf + sizeof(buf) - 1;
-
-  int fd;
-  NO_INTR(fd = open(path, O_RDONLY));
-  if (fd < 0) {
-    return false;
-  }
-
-  char* sbuf = buf; // note, initial value could be nullptr, but
-                    // memmove actually insists to get non-null
-                    // arguments (even when count is 0)
-  char* ebuf = sbuf;
-
-  bool eof = false;
-
-  for (;;) {
-    char* nextline = static_cast<char*>(memchr(sbuf, '\n', ebuf - sbuf));
-
-    if (nextline != nullptr) {
-      RAW_CHECK(nextline < ebuf, "BUG");
-
-      *nextline = 0; // Turn newline into '\0'.
-
-      if (!body(sbuf, nextline)) {
-        break;
-      }
-
-      sbuf = nextline + 1;
-      continue;
-    }
-
-    int count = ebuf - sbuf;
-
-    // TODO: what if we face way too long line ?
-
-    if (eof) {
-      if (count == 0) {
-        break; // done
-      }
-
-      // Last read ended up without trailing newline. Lets add
-      // it. Note, we left one byte margin above, so we're able to
-      // write this and not get past end of buf.
-      *ebuf++ = '\n';
-      continue;
-    }
-
-    // Move the current text to the start of the buffer
-    memmove(buf, sbuf, count);
-    sbuf = buf;
-    ebuf = sbuf + count;
-
-    int nread;
-    NO_INTR(nread = read(fd, ebuf, buf_end - ebuf));
-
-    // Read failures are not expected, but lets not crash if this
-    // happens in non-debug mode.
-    DCHECK_GE(nread, 0);
-    if (nread < 0) {
-      nread = 0;
-    }
-
-    if (nread == 0) {
-      eof = true;
-    }
-    ebuf += nread;
-    // Retry memchr above.
-  }
-
-  close(fd);
-  return true;
-}
-
-inline
 bool DoIterateLinux(const char* path, void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
-  // TODO: line_end is unused ?
-  return ForEachLine(
+  return DoForEachLine(
     path,
     [&] (char* line_start, char* line_end) {
       unsigned filename_offset;
@@ -338,8 +260,6 @@ bool DoIterateLinux(const char* path, void (*body)(const ProcMapping& mapping, v
                              flags_tmp,
                              &mapping.offset, &mapping.inode,
                              &filename_offset)) {
-        int c = line_end - line_start;
-        fprintf(stderr, "bad line %d:\n%.*s\n----\n", c, c, line_start);
         return false;
       }
 
@@ -349,11 +269,11 @@ bool DoIterateLinux(const char* path, void (*body)(const ProcMapping& mapping, v
       return true;
     });
 }
+#endif  // __linux__ || __NetBSD__
 
 #if defined(__QNXNTO__)
-inline
 bool DoIterateQNX(void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
-  return ForEachLine(
+  return DoForEachLine(
     "/proc/self/pmap",
     [&] (char* line_start, char* line_end) {
       ProcMapping mapping;
@@ -408,41 +328,7 @@ bool DoIterateQNX(void (*body)(const ProcMapping& mapping, void* arg), void* arg
 }
 #endif
 
-inline
-bool DoIterateFreeBSD(void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
-  return ForEachLine(
-    "/proc/curproc/map",
-    [&] (char* line_start, char* line_end) {
-      if (line_start == line_end) {
-        return true; // freebsd is weird
-      }
-      ProcMapping mapping;
-      memset(&mapping, 0, sizeof(mapping));
-
-      unsigned filename_offset;
-      char flags_tmp[10];
-      // start end resident privateresident obj(?) prot refcnt shadowcnt
-      // flags copy_on_write needs_copy type filename:
-      // 0x8048000 0x804a000 2 0 0xc104ce70 r-x 1 0 0x0 COW NC vnode /bin/cat
-      if (sscanf(line_start, "0x%" SCNx64 " 0x%" SCNx64 " %*d %*d %*p %3s %*d %*d 0x%*x %*s %*s %*s %n",
-                 &mapping.start,
-                 &mapping.end,
-                 flags_tmp,
-                 &filename_offset) != 3) {
-        return false;
-      }
-
-      mapping.flags = flags_tmp;
-      mapping.filename = line_start + filename_offset;
-
-      body(mapping, arg);
-
-      return true;
-    });
-}
-
 #if defined(__sun__)
-inline
 bool DoIterateSolaris(void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
   int fd;
   NO_INTR(fd = open("/proc/self/map", O_RDONLY));
@@ -498,7 +384,6 @@ bool DoIterateSolaris(void (*body)(const ProcMapping& mapping, void* arg), void*
 #endif
 
 #if defined(PLATFORM_WINDOWS)
-inline
 bool DoIterateWindows(void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
   HANDLE snapshot = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE |
                                              TH32CS_SNAPMODULE32,
@@ -541,7 +426,6 @@ bool DoIterateWindows(void (*body)(const ProcMapping& mapping, void* arg), void*
 #endif  // defined(PLATFORM_WINDOWS)
 
 #if defined(__MACH__)
-
 // A templatized helper function instantiated for Mach (OS X) only.
 // It can handle finding info for both 32 bits and 64 bits.
 // Returns true if it successfully handled the hdr, false else.
@@ -550,7 +434,7 @@ template<uint32_t kMagic, uint32_t kLCSegment,
 bool NextExtMachHelper(const mach_header* hdr,
                               int current_image, int current_load_cmd,
                               uint64_t *start, uint64_t *end, const char **flags,
-                              uint64_t *offset, int64_t *inode, const char **filename) {
+                              uint64_t *offset, const char **filename) {
   static char kDefaultPerms[5] = "r-xp";
   if (hdr->magic != kMagic)
     return false;
@@ -565,7 +449,6 @@ bool NextExtMachHelper(const mach_header* hdr,
     if (end) *end = sc->vmaddr + sc->vmsize + dlloff;
     if (flags) *flags = kDefaultPerms;  // can we do better?
     if (offset) *offset = sc->fileoff;
-    if (inode) *inode = 0;
     if (filename)
       *filename = const_cast<char*>(_dyld_get_image_name(current_image));
     return true;
@@ -596,7 +479,7 @@ reenter:
             hdr, current_image, current_load_cmd,
             &mapping.start, &mapping.end,
             &mapping.flags,
-            &mapping.offset, &mapping.inode, &mapping.filename)) {
+            &mapping.offset, &mapping.filename)) {
         body(mapping, arg);
         goto reenter;
       }
@@ -606,7 +489,7 @@ reenter:
             hdr, current_image, current_load_cmd,
             &mapping.start, &mapping.end,
             &mapping.flags,
-            &mapping.offset, &mapping.inode, &mapping.filename)) {
+            &mapping.offset, &mapping.filename)) {
         body(mapping, arg);
         goto reenter;
       }
@@ -621,7 +504,7 @@ reenter:
 
 void FormatLine(tcmalloc::GenericWriter* writer,
                 uint64_t start, uint64_t end, const char *flags,
-                uint64_t offset, int64_t inode,
+                uint64_t offset, uint64_t inode,
                 const char *filename, dev_t dev) {
   // We assume 'flags' looks like 'rwxp' or 'rwx'.
   char r = (flags && flags[0] == 'r') ? 'r' : '-';
@@ -631,7 +514,7 @@ void FormatLine(tcmalloc::GenericWriter* writer,
   char p = (flags && flags[0] && flags[1] && flags[2] && flags[3] != 'p')
       ? '-' : 'p';
 
-  writer->AppendF("%08" PRIx64 "-%08" PRIx64 " %c%c%c%c %08" PRIx64 " %02x:%02x %-11" PRId64,
+  writer->AppendF("%08" PRIx64 "-%08" PRIx64 " %c%c%c%c %08" PRIx64 " %02x:%02x %" PRIu64 " ",
                   start, end, r,w,x,p, offset,
                   static_cast<int>(dev/256), static_cast<int>(dev%256),
                   inode);
@@ -639,10 +522,68 @@ void FormatLine(tcmalloc::GenericWriter* writer,
   writer->AppendStr("\n");
 }
 
-}  // namespace
+}  // anonymous namespace
 
 bool DoForEachProcMapping(void (*body)(const ProcMapping& mapping, void* arg), void* arg) {
-#if defined(PLATFORM_WINDOWS)
+#if defined TCMALLOC_WANT_DL_ITERATE_PHDR
+  auto callback = [&] (struct dl_phdr_info *info, size_t _size) -> int {
+    ProcMapping mapping;
+    memset(&mapping, 0, sizeof(mapping));
+    uintptr_t addr = info->dlpi_addr;
+    if (!info->dlpi_name || info->dlpi_name[0] == 0) {
+      mapping.filename = tcmalloc::GetProgramInvocationName();
+      if (!mapping.filename) {
+        mapping.filename = "";
+      }
+    } else {
+      mapping.filename = info->dlpi_name;
+    }
+    for (int i = 0; i < info->dlpi_phnum; i++) {
+      auto phdr = info->dlpi_phdr + i;
+      if (phdr->p_type != PT_LOAD) {
+        continue;
+      }
+      uint64_t vaddr = phdr->p_vaddr + addr;
+      uint64_t align = phdr->p_align;
+      uint64_t offset = phdr->p_offset;
+      uint64_t size = phdr->p_filesz;
+      uint64_t start_adj = vaddr & (align - 1); // amount we need to round down to align start
+      uint64_t end_adj = (~(vaddr + size) + 1) & (align - 1); // likewise, but for end of mapping
+
+      vaddr -= start_adj;
+      offset -= start_adj;
+      size += start_adj + end_adj;
+      CHECK_EQ((size & (align - 1)), 0);
+
+      char flags[5] = "---p";
+      auto fl = phdr->p_flags;
+      if ((fl & PF_R)) {
+        flags[0] = 'r';
+      }
+      if ((fl & PF_W)) {
+        flags[1] = 'w';
+      }
+      if ((fl & PF_X)) {
+        flags[2] = 'x';
+      }
+
+      mapping.start = vaddr;
+      mapping.end = vaddr + size;
+      mapping.flags = flags;
+      mapping.offset = offset;
+      mapping.inode = 0;
+
+      body(mapping, arg);
+    }
+
+    return 0;
+  };
+
+  tcmalloc::FunctionRef<int(struct dl_phdr_info *, size_t)> ref{callback};
+  (void)dl_iterate_phdr(ref.fn, ref.data);
+
+  return true;
+#elif defined(PLATFORM_WINDOWS)
   return DoIterateWindows(body, arg);
 #elif defined(__MACH__)
   return DoIterateOSX(body, arg);
@@ -652,8 +593,10 @@ bool DoForEachProcMapping(void (*body)(const ProcMapping& mapping, void* arg), v
   return DoIterateQNX(body, arg);
 #elif defined(__FreeBSD__)
   return DoIterateFreeBSD(body, arg);
-#else
+#elif defined(__linux__) || defined(__NetBSD__)
   return DoIterateLinux("/proc/self/maps", body, arg);
+#else
+  return false;
 #endif
 }
 
