@@ -75,7 +75,7 @@
 #include "base/threading.h"
 #include "malloc_backtrace.h"
 #include "malloc_hook-inl.h"
-#include "maybe_emergency_malloc.h"
+#include "maybe_emergency_malloc.h" // IWYU pragma: keep
 #include "safe_strerror.h"
 #include "symbolize.h"
 
@@ -198,17 +198,14 @@ class FreeQueue {
 };
 
 struct MallocBlockQueueEntry {
-  MallocBlockQueueEntry() : block(NULL), size(0),
+  MallocBlockQueueEntry() : block(nullptr), size(0),
                             num_deleter_pcs(0) {}
   MallocBlockQueueEntry(MallocBlock* b, size_t s) : block(b), size(s) {
     if (FLAGS_max_free_queue_size != 0 && b != nullptr) {
-      // Adjust the number of frames to skip (4) if you change the
-      // location of this call.
-      num_deleter_pcs =
-        MallocHook::GetCallerStackTrace(
-          deleter_pcs,
-          sizeof(deleter_pcs) / sizeof(deleter_pcs[0]),
-          4);
+      num_deleter_pcs = tcmalloc::GrabBacktrace(
+        deleter_pcs,
+        sizeof(deleter_pcs) / sizeof(deleter_pcs[0]),
+        1);
       deleter_threadid = tcmalloc::SelfThreadId();
     } else {
       num_deleter_pcs = 0;
@@ -425,8 +422,8 @@ class MallocBlock {
   void CheckLocked(int type) const {
     int map_type = 0;
     const int* found_type =
-      alloc_map_ != NULL ? alloc_map_->Find(data_addr()) : NULL;
-    if (found_type == NULL) {
+      alloc_map_ != nullptr ? alloc_map_->Find(data_addr()) : nullptr;
+    if (found_type == nullptr) {
       RAW_LOG(FATAL, "memory allocation bug: object at %p "
                      "has never been allocated", data_addr());
     } else {
@@ -517,9 +514,9 @@ class MallocBlock {
     static size_t max_size_t = ~0;
     if (size > max_size_t - sizeof(MallocBlock)) {
       RAW_LOG(ERROR, "Massive size passed to malloc: %zu", size);
-      return NULL;
+      return nullptr;
     }
-    MallocBlock* b = NULL;
+    MallocBlock* b = nullptr;
     const bool use_malloc_page_fence = FLAGS_malloc_page_fence;
     const bool malloc_page_fence_readable = FLAGS_malloc_page_fence_readable;
 #ifdef HAVE_MMAP
@@ -529,10 +526,10 @@ class MallocBlock {
       size_t sz = real_mmapped_size(size);
       int pagesize = getpagesize();
       int num_pages = (sz + pagesize - 1) / pagesize + 1;
-      char* p = (char*) mmap(NULL, num_pages * pagesize, PROT_READ|PROT_WRITE,
+      char* p = (char*) mmap(nullptr, num_pages * pagesize, PROT_READ|PROT_WRITE,
                              MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
       if (p == MAP_FAILED) {
-        // If the allocation fails, abort rather than returning NULL to
+        // If the allocation fails, abort rather than returning nullptr to
         // malloc. This is because in most cases, the program will run out
         // of memory in this mode due to tremendous amount of wastage. There
         // is no point in propagating the error elsewhere.
@@ -556,8 +553,8 @@ class MallocBlock {
     // It would be nice to output a diagnostic on allocation failure
     // here, but logging (other than FATAL) requires allocating
     // memory, which could trigger a nasty recursion. Instead, preserve
-    // malloc semantics and return NULL on failure.
-    if (b != NULL) {
+    // malloc semantics and return nullptr on failure.
+    if (b != nullptr) {
       b->magic1_ = use_malloc_page_fence ? kMagicMMap : kMagicMalloc;
       b->Initialize(size, type);
     }
@@ -605,11 +602,11 @@ class MallocBlock {
     int num_entries = 0;
     MallocBlockQueueEntry new_entry(b, size);
     free_queue_lock_.Lock();
-    if (free_queue_ == NULL)
+    if (free_queue_ == nullptr)
       free_queue_ = new FreeQueue<MallocBlockQueueEntry>;
     RAW_CHECK(!free_queue_->Full(), "Free queue mustn't be full!");
 
-    if (b != NULL) {
+    if (b != nullptr) {
       free_queue_size_ += size + sizeof(MallocBlockQueueEntry);
       free_queue_->Push(new_entry);
     }
@@ -699,25 +696,11 @@ class MallocBlock {
       TracePrintf(STDERR_FILENO, "Deleted by thread %zx\n",
                   queue_entry.deleter_threadid);
 
-      // We don't want to allocate or deallocate memory here, so we use
-      // placement-new.  It's ok that we don't destroy this, since we're
-      // just going to error-exit below anyway.
-      tcmalloc::StaticStorage<SymbolTable> tablebuf;
-      SymbolTable* symbolization_table = tablebuf.Construct();
-      for (int i = 0; i < queue_entry.num_deleter_pcs; i++) {
-        // Symbolizes the previous address of pc because pc may be in the
-        // next function.  This may happen when the function ends with
-        // a call to a function annotated noreturn (e.g. CHECK).
-        char *pc = reinterpret_cast<char*>(queue_entry.deleter_pcs[i]);
-        symbolization_table->Add(pc - 1);
-      }
-      if (FLAGS_symbolize_stacktrace)
-        symbolization_table->Symbolize();
-      for (int i = 0; i < queue_entry.num_deleter_pcs; i++) {
-        char *pc = reinterpret_cast<char*>(queue_entry.deleter_pcs[i]);
-        TracePrintf(STDERR_FILENO, "    @ %p %s\n",
-                    pc, symbolization_table->GetSymbol(pc - 1));
-      }
+      ThreadCachePtr::WithStacktraceScope([&] (bool stacktrace_allowed) {
+        tcmalloc::DumpStackTraceToStderr(queue_entry.deleter_pcs, queue_entry.num_deleter_pcs,
+                                         FLAGS_symbolize_stacktrace,
+                                         "    @ ");
+      });
     } else {
       RAW_LOG(ERROR,
               "Skipping the printing of the deleter's stack!  Its stack was "
@@ -845,7 +828,7 @@ class MallocBlock {
 
 void DanglingWriteChecker() {
   // Clear out the remaining free queue to check for dangling writes.
-  MallocBlock::ProcessFreeQueue(NULL, 0, 0);
+  MallocBlock::ProcessFreeQueue(nullptr, 0, 0);
 }
 
 // ========================================================================= //
@@ -860,14 +843,14 @@ const char* const MallocBlock::kAllocName[] = {
   "malloc",
   "new",
   "new []",
-  NULL,
+  nullptr,
 };
 
 const char* const MallocBlock::kDeallocName[] = {
   "free",
   "delete",
   "delete []",
-  NULL,
+  nullptr,
 };
 
 int MallocBlock::stats_blocks_;
@@ -987,7 +970,7 @@ static int TraceFd() {
     }
     // Add a header to the log.
     TracePrintf(trace_fd, "Trace started: %lu\n",
-                static_cast<unsigned long>(time(NULL)));
+                static_cast<unsigned long>(time(nullptr)));
     TracePrintf(trace_fd,
                 "func\tsize\tptr\tthread_id\tstack pcs for tools/symbolize\n");
   }
@@ -1051,7 +1034,7 @@ static inline void* DebugAllocate(size_t size, int type) {
   if (size == 0) size = 1;
 #endif
   MallocBlock* ptr = MallocBlock::Allocate(size, type);
-  if (ptr == NULL)  return NULL;
+  if (ptr == nullptr)  return nullptr;
   MALLOC_TRACE("malloc", size, ptr->data_addr());
   return ptr->data_addr();
 }
@@ -1067,11 +1050,19 @@ static inline void DebugDeallocate(void* ptr, int type, size_t given_size) {
   if (ptr)  MallocBlock::FromRawPointer(ptr)->Deallocate(type, given_size);
 }
 
-class ATTRIBUTE_HIDDEN DebugTestingPortal : public TestingPortalImpl {
+class ATTRIBUTE_VISIBILITY_HIDDEN DebugTestingPortal : public TestingPortalImpl {
 public:
   ~DebugTestingPortal() override = default;
   bool IsDebuggingMalloc() override { return true; }
   int32_t& GetMaxFreeQueueSize() override { return FLAGS_max_free_queue_size; }
+  uint32_t GetSizeClass(void*) override { return 0; }
+  void* RunReallocWithCallback(
+    void* old_ptr, size_t new_size,
+    void (*invalid_free_fn)(void*),
+    size_t (*invalid_get_size_fn)(const void*)) override {
+    RAW_LOG(FATAL, "RunReallocWithCallback is not implemented for debugallocation");
+    return nullptr;
+  }
 };
 
 // ========================================================================= //
@@ -1134,6 +1125,9 @@ class DebugMallocImplementation : public TCMallocImplementation {
     if (p) {
       RAW_CHECK(GetOwnership(p) != MallocExtension::kNotOwned,
                 "ptr not allocated by tcmalloc");
+      if (tcmalloc::IsEmergencyPtr(p)) {
+        return tcmalloc::EmergencyAllocatedSize(p);
+      }
       return MallocBlock::FromRawPointer(p)->actual_data_size(p);
     }
     return 0;
@@ -1141,7 +1135,7 @@ class DebugMallocImplementation : public TCMallocImplementation {
 
   virtual MallocExtension::Ownership GetOwnership(const void* p) {
     if (!p) {
-      // nobody owns NULL
+      // nobody owns nullptr
       return MallocExtension::kNotOwned;
     }
 
@@ -1168,11 +1162,15 @@ class DebugMallocImplementation : public TCMallocImplementation {
       return rv;
     }
 
+    if (tcmalloc::IsEmergencyPtr(p)) {
+      return kOwned;
+    }
+
     const MallocBlock* mb = MallocBlock::FromRawPointer(p);
     return TCMallocImplementation::GetOwnership(mb);
   }
 
-  virtual void GetFreeListSizes(vector<MallocExtension::FreeListInfo>* v) {
+  virtual void GetFreeListSizes(std::vector<MallocExtension::FreeListInfo>* v) {
     static const char* kDebugFreeQueue = "debug.free_queue";
 
     TCMallocImplementation::GetFreeListSizes(v);
@@ -1180,7 +1178,7 @@ class DebugMallocImplementation : public TCMallocImplementation {
     MallocExtension::FreeListInfo i;
     i.type = kDebugFreeQueue;
     i.min_object_size = 0;
-    i.max_object_size = numeric_limits<size_t>::max();
+    i.max_object_size = std::numeric_limits<size_t>::max();
     i.total_bytes_free = MallocBlock::FreeQueueSize();
     v->push_back(i);
   }
@@ -1220,7 +1218,7 @@ static void *retry_debug_allocate(void *arg) {
 // from the logic of calling the new-handler.
 inline void* debug_cpp_alloc(size_t size, int new_type, bool nothrow) {
   void* p = DebugAllocate(size, new_type);
-  if (p != NULL) {
+  if (p != nullptr) {
     return p;
   }
   struct debug_alloc_retry_data data;
@@ -1232,7 +1230,7 @@ inline void* debug_cpp_alloc(size_t size, int new_type, bool nothrow) {
 
 inline void* do_debug_malloc_or_debug_cpp_alloc(size_t size) {
   void* p = DebugAllocate(size, MallocBlock::kMallocType);
-  if (p != NULL) {
+  if (p != nullptr) {
     return p;
   }
   struct debug_alloc_retry_data data;
@@ -1259,35 +1257,41 @@ static void force_frame() {
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_malloc(size_t size) PERFTOOLS_NOTHROW {
   void* ptr = do_debug_malloc_or_debug_cpp_alloc(size);
-  MallocHook::InvokeNewHook(ptr, size);
+  tcmalloc::InvokeNewHook(ptr, size);
   return ptr;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_free(void* ptr) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(ptr);
+  tcmalloc::InvokeDeleteHook(ptr);
   DebugDeallocate(ptr, MallocBlock::kMallocType, 0);
   force_frame();
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_free_sized(void *ptr, size_t size) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(ptr);
+  tcmalloc::InvokeDeleteHook(ptr);
   DebugDeallocate(ptr, MallocBlock::kMallocType, size);
   force_frame();
 }
 
+extern "C" PERFTOOLS_DLL_DECL void tc_free_aligned_sized(void* ptr, size_t align, size_t size) PERFTOOLS_NOTHROW {
+  tcmalloc::InvokeDeleteHook(ptr);
+  DebugDeallocate(ptr, MallocBlock::kMallocType, 0);
+  force_frame();
+}
+
 extern "C" PERFTOOLS_DLL_DECL void* tc_calloc(size_t count, size_t size) PERFTOOLS_NOTHROW {
   // Overflow check
   const size_t total_size = count * size;
-  if (size != 0 && total_size / size != count) return NULL;
+  if (size != 0 && total_size / size != count) return nullptr;
 
   void* block = do_debug_malloc_or_debug_cpp_alloc(total_size);
   if (block)  memset(block, 0, total_size);
-  MallocHook::InvokeNewHook(block, total_size);
+  tcmalloc::InvokeNewHook(block, total_size);
   return block;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_cfree(void* ptr) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(ptr);
+  tcmalloc::InvokeDeleteHook(ptr);
   DebugDeallocate(ptr, MallocBlock::kMallocType, 0);
   force_frame();
 }
@@ -1295,11 +1299,11 @@ extern "C" PERFTOOLS_DLL_DECL void tc_cfree(void* ptr) PERFTOOLS_NOTHROW {
 extern "C" PERFTOOLS_DLL_DECL void* tc_realloc(void* ptr, size_t size) PERFTOOLS_NOTHROW {
   if (ptr == nullptr) {
     ptr = do_debug_malloc_or_debug_cpp_alloc(size);
-    MallocHook::InvokeNewHook(ptr, size);
+    tcmalloc::InvokeNewHook(ptr, size);
     return ptr;
   }
   if (size == 0) {
-    MallocHook::InvokeDeleteHook(ptr);
+    tcmalloc::InvokeDeleteHook(ptr);
     DebugDeallocate(ptr, MallocBlock::kMallocType, 0);
     return nullptr;
   }
@@ -1319,8 +1323,8 @@ extern "C" PERFTOOLS_DLL_DECL void* tc_realloc(void* ptr, size_t size) PERFTOOLS
   size_t old_size = old->actual_data_size(ptr);
 
   memcpy(p->data_addr(), ptr, (old_size < size) ? old_size : size);
-  MallocHook::InvokeDeleteHook(ptr);
-  MallocHook::InvokeNewHook(p->data_addr(), size);
+  tcmalloc::InvokeDeleteHook(ptr);
+  tcmalloc::InvokeNewHook(p->data_addr(), size);
   DebugDeallocate(ptr, MallocBlock::kMallocType, 0);
   MALLOC_TRACE("realloc", p->actual_data_size(p->data_addr()), p->data_addr());
   return p->data_addr();
@@ -1328,8 +1332,8 @@ extern "C" PERFTOOLS_DLL_DECL void* tc_realloc(void* ptr, size_t size) PERFTOOLS
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_new(size_t size) {
   void* ptr = debug_cpp_alloc(size, MallocBlock::kNewType, false);
-  MallocHook::InvokeNewHook(ptr, size);
-  if (ptr == NULL) {
+  tcmalloc::InvokeNewHook(ptr, size);
+  if (ptr == nullptr) {
     RAW_LOG(FATAL, "Unable to allocate %zu bytes: new failed.", size);
   }
   return ptr;
@@ -1337,18 +1341,18 @@ extern "C" PERFTOOLS_DLL_DECL void* tc_new(size_t size) {
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_new_nothrow(size_t size, const std::nothrow_t&) PERFTOOLS_NOTHROW {
   void* ptr = debug_cpp_alloc(size, MallocBlock::kNewType, true);
-  MallocHook::InvokeNewHook(ptr, size);
+  tcmalloc::InvokeNewHook(ptr, size);
   return ptr;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_delete(void* p) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kNewType, 0);
   force_frame();
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_delete_sized(void* p, size_t size) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kNewType, size);
   force_frame();
 }
@@ -1356,15 +1360,15 @@ extern "C" PERFTOOLS_DLL_DECL void tc_delete_sized(void* p, size_t size) PERFTOO
 // Some STL implementations explicitly invoke this.
 // It is completely equivalent to a normal delete (delete never throws).
 extern "C" PERFTOOLS_DLL_DECL void tc_delete_nothrow(void* p, const std::nothrow_t&) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kNewType, 0);
   force_frame();
 }
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_newarray(size_t size) {
   void* ptr = debug_cpp_alloc(size, MallocBlock::kArrayNewType, false);
-  MallocHook::InvokeNewHook(ptr, size);
-  if (ptr == NULL) {
+  tcmalloc::InvokeNewHook(ptr, size);
+  if (ptr == nullptr) {
     RAW_LOG(FATAL, "Unable to allocate %zu bytes: new[] failed.", size);
   }
   return ptr;
@@ -1373,18 +1377,18 @@ extern "C" PERFTOOLS_DLL_DECL void* tc_newarray(size_t size) {
 extern "C" PERFTOOLS_DLL_DECL void* tc_newarray_nothrow(size_t size, const std::nothrow_t&)
     PERFTOOLS_NOTHROW {
   void* ptr = debug_cpp_alloc(size, MallocBlock::kArrayNewType, true);
-  MallocHook::InvokeNewHook(ptr, size);
+  tcmalloc::InvokeNewHook(ptr, size);
   return ptr;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_deletearray(void* p) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kArrayNewType, 0);
   force_frame();
 }
 
 extern "C" PERFTOOLS_DLL_DECL void tc_deletearray_sized(void* p, size_t size) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kArrayNewType, size);
   force_frame();
 }
@@ -1392,7 +1396,7 @@ extern "C" PERFTOOLS_DLL_DECL void tc_deletearray_sized(void* p, size_t size) PE
 // Some STL implementations explicitly invoke this.
 // It is completely equivalent to a normal delete (delete never throws).
 extern "C" PERFTOOLS_DLL_DECL void tc_deletearray_nothrow(void* p, const std::nothrow_t&) PERFTOOLS_NOTHROW {
-  MallocHook::InvokeDeleteHook(p);
+  tcmalloc::InvokeDeleteHook(p);
   DebugDeallocate(p, MallocBlock::kArrayNewType, 0);
   force_frame();
 }
@@ -1407,7 +1411,7 @@ static void *do_debug_memalign(size_t alignment, size_t size, int type) {
   // Allocate "alignment-1" extra bytes to ensure alignment is possible, and
   // a further data_offset bytes for an additional fake header.
   size_t extra_bytes = data_offset + alignment - 1;
-  if (size + extra_bytes < size) return NULL;         // Overflow
+  if (size + extra_bytes < size) return nullptr;         // Overflow
   p = DebugAllocate(size + extra_bytes, type);
   if (p != 0) {
     intptr_t orig_p = reinterpret_cast<intptr_t>(p);
@@ -1448,7 +1452,7 @@ void* do_debug_memalign_or_debug_cpp_memalign(size_t align,
                                                      bool from_operator,
                                                      bool nothrow) {
   void* p = do_debug_memalign(align, size, type);
-  if (p != NULL) {
+  if (p != nullptr) {
     return p;
   }
 
@@ -1462,7 +1466,7 @@ void* do_debug_memalign_or_debug_cpp_memalign(size_t align,
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_memalign(size_t align, size_t size) PERFTOOLS_NOTHROW {
   void *p = do_debug_memalign_or_debug_cpp_memalign(align, size, MallocBlock::kMallocType, false, true);
-  MallocHook::InvokeNewHook(p, size);
+  tcmalloc::InvokeNewHook(p, size);
   return p;
 }
 
@@ -1476,8 +1480,8 @@ extern "C" PERFTOOLS_DLL_DECL int tc_posix_memalign(void** result_ptr, size_t al
   }
 
   void* result = do_debug_memalign_or_debug_cpp_memalign(align, size, MallocBlock::kMallocType, false, true);
-  MallocHook::InvokeNewHook(result, size);
-  if (result == NULL) {
+  tcmalloc::InvokeNewHook(result, size);
+  if (result == nullptr) {
     return ENOMEM;
   } else {
     *result_ptr = result;
@@ -1488,7 +1492,7 @@ extern "C" PERFTOOLS_DLL_DECL int tc_posix_memalign(void** result_ptr, size_t al
 extern "C" PERFTOOLS_DLL_DECL void* tc_valloc(size_t size) PERFTOOLS_NOTHROW {
   // Allocate >= size bytes starting on a page boundary
   void *p = do_debug_memalign_or_debug_cpp_memalign(getpagesize(), size, MallocBlock::kMallocType, false, true);
-  MallocHook::InvokeNewHook(p, size);
+  tcmalloc::InvokeNewHook(p, size);
   return p;
 }
 
@@ -1501,19 +1505,19 @@ extern "C" PERFTOOLS_DLL_DECL void* tc_pvalloc(size_t size) PERFTOOLS_NOTHROW {
     size = pagesize;   // http://man.free4web.biz/man3/libmpatrol.3.html
   }
   void *p = do_debug_memalign_or_debug_cpp_memalign(pagesize, size, MallocBlock::kMallocType, false, true);
-  MallocHook::InvokeNewHook(p, size);
+  tcmalloc::InvokeNewHook(p, size);
   return p;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_new_aligned(size_t size, std::align_val_t align) {
   void* result = do_debug_memalign_or_debug_cpp_memalign(static_cast<size_t>(align), size, MallocBlock::kNewType, true, false);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_new_aligned_nothrow(size_t size, std::align_val_t align, const std::nothrow_t&) PERFTOOLS_NOTHROW {
   void* result = do_debug_memalign_or_debug_cpp_memalign(static_cast<size_t>(align), size, MallocBlock::kNewType, true, true);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
 
@@ -1536,13 +1540,13 @@ extern "C" PERFTOOLS_DLL_DECL void tc_delete_aligned_nothrow(void* p, std::align
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_newarray_aligned(size_t size, std::align_val_t align) {
   void* result = do_debug_memalign_or_debug_cpp_memalign(static_cast<size_t>(align), size, MallocBlock::kArrayNewType, true, false);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_newarray_aligned_nothrow(size_t size, std::align_val_t align, const std::nothrow_t& nt) PERFTOOLS_NOTHROW {
   void* result = do_debug_memalign_or_debug_cpp_memalign(static_cast<size_t>(align), size, MallocBlock::kArrayNewType, true, true);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
 
@@ -1590,6 +1594,6 @@ extern "C" PERFTOOLS_DLL_DECL size_t tc_malloc_size(void* ptr) PERFTOOLS_NOTHROW
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_malloc_skip_new_handler(size_t size) PERFTOOLS_NOTHROW {
   void* result = DebugAllocate(size, MallocBlock::kMallocType);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
