@@ -124,7 +124,7 @@ struct GetStackImplementation {
 #define HAVE_GST_libunwind
 #endif // USE_LIBUNWIND
 
-#if defined(__GNUC__) && (defined(__x86_64__) || defined(__i386__) || defined(__aarch64__) || defined(__riscv) || defined(__arm__))
+#if !defined(_WIN32) && defined(__GNUC__) && (defined(__x86_64__) || defined(__i386__) || defined(__aarch64__) || defined(__riscv) || defined(__arm__))
 // NOTE: legacy 32-bit arm works fine with recent clangs, but is broken in gcc: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=92172
 #define STACKTRACE_INL_HEADER "stacktrace_generic_fp-inl.h"
 #define GST_SUFFIX generic_fp
@@ -144,6 +144,7 @@ struct GetStackImplementation {
 #define HAVE_GST_generic_fp_unsafe
 #endif
 
+#if !defined(TCMALLOC_DISABLE_PPC_FRAME_POINTER_BACKTRACER)
 #if defined(__ppc__) || defined(__PPC__)
 #if defined(__linux__)
 #define STACKTRACE_INL_HEADER "stacktrace_powerpc-linux-inl.h"
@@ -156,6 +157,7 @@ struct GetStackImplementation {
 #undef STACKTRACE_INL_HEADER
 #define HAVE_GST_ppc
 #endif
+#endif  // !TCMALLOC_DISABLE_PPC_FRAME_POINTER_BACKTRACER
 
 #if defined(__arm__)
 #define STACKTRACE_INL_HEADER "stacktrace_arm-inl.h"
