@@ -41,11 +41,11 @@
 // Anything that should only be defined once should be here:
 
 #include <stdint.h>   // for uintptr_t
-#include <stdlib.h>   // for NULL
+#include <stdlib.h>
 #include <gperftools/stacktrace.h>
 
 // Given a pointer to a stack frame, locate and return the calling
-// stackframe, or return NULL if no stackframe can be found. Perform sanity
+// stackframe, or return nullptr if no stackframe can be found. Perform sanity
 // checks (the strictness of which is controlled by the boolean parameter
 // "STRICT_UNWINDING") to reduce the chance that a bad pointer is returned.
 template<bool STRICT_UNWINDING>
@@ -57,18 +57,18 @@ static void **NextStackFrame(void **old_sp) {
   if (STRICT_UNWINDING) {
     // With the stack growing downwards, older stack frame must be
     // at a greater address that the current one.
-    if (new_sp <= old_sp) return NULL;
+    if (new_sp <= old_sp) return nullptr;
     // Assume stack frames larger than 100,000 bytes are bogus.
-    if ((uintptr_t)new_sp - (uintptr_t)old_sp > 100000) return NULL;
+    if ((uintptr_t)new_sp - (uintptr_t)old_sp > 100000) return nullptr;
   } else {
     // In the non-strict mode, allow discontiguous stack frames.
     // (alternate-signal-stacks for example).
-    if (new_sp == old_sp) return NULL;
+    if (new_sp == old_sp) return nullptr;
     // And allow frames upto about 1MB.
     if ((new_sp > old_sp)
-        && ((uintptr_t)new_sp - (uintptr_t)old_sp > 1000000)) return NULL;
+        && ((uintptr_t)new_sp - (uintptr_t)old_sp > 1000000)) return nullptr;
   }
-  if ((uintptr_t)new_sp & (sizeof(void *) - 1)) return NULL;
+  if ((uintptr_t)new_sp & (sizeof(void *) - 1)) return nullptr;
   return new_sp;
 }
 
@@ -98,7 +98,7 @@ int GET_STACK_TRACE_OR_FRAMES {
   // different asm syntax.  I don't know quite the best way to discriminate
   // systems using the old as from the new one; I've gone with __APPLE__.
   // TODO(csilvers): use autoconf instead, to look for 'as --version' == 1 or 2
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(_AIX)
   __asm__ volatile ("mr %0,1" : "=r" (sp));
 #else
   __asm__ volatile ("mr %0,r1" : "=r" (sp));
