@@ -74,12 +74,11 @@
 //      pagemap[q] == s
 //      pagemap[p+1..q-1] are undefined
 //      pagemap[p-1] and pagemap[q+1] are defined:
-//         NULL if the corresponding page is not yet in the address space.
+//         nullptr if the corresponding page is not yet in the address space.
 //         Otherwise it points to a Span.  This span may be free
 //         or allocated.  If free, it is in one of pageheap's freelist.
 //
 // TODO: Bias reclamation to larger addresses
-// TODO: implement mallinfo/mallopt
 // TODO: Better testing
 //
 // 9/28/2003 (new page-level allocator replaces ptmalloc2):
@@ -95,7 +94,7 @@
 
 #include <errno.h>                      // for ENOMEM, EINVAL, errno
 #include <stdint.h>
-#include <stddef.h>                     // for size_t, NULL
+#include <stddef.h>                     // for size_t
 #include <stdlib.h>                     // for getenv
 #include <string.h>                     // for strcmp, memset, strlen, etc
 #ifdef HAVE_UNISTD_H
@@ -117,7 +116,7 @@
 #include "common.h"            // for StackTrace, kPageShift, etc
 #include "internal_logging.h"  // for ASSERT, TCMalloc_Printer, etc
 #include "linked_list.h"       // for SLL_SetNext
-#include "malloc_hook-inl.h"       // for MallocHook::InvokeNewHook, etc
+#include "malloc_hook-inl.h"       // for tcmalloc::InvokeNewHook, etc
 #include "page_heap.h"         // for PageHeap, PageHeap::Stats
 #include "page_heap_allocator.h"  // for PageHeapAllocator
 #include "span.h"              // for Span, DLL_Prepend, etc
@@ -129,7 +128,7 @@
 #include "thread_cache_ptr.h"
 
 #include "malloc_backtrace.h"
-#include "maybe_emergency_malloc.h"
+#include "maybe_emergency_malloc.h" // IWYU pragma: keep
 #include "testing_portal.h"
 
 #if (defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)) && !defined(WIN32_OVERRIDE_ALLOCATORS)
@@ -139,19 +138,11 @@
 // Some windows file somewhere (at least on cygwin) #define's small (!)
 #undef small
 
-using std::max;
-using std::min;
-using std::numeric_limits;
-using std::vector;
-
 #include "libc_override.h"
 
-using tcmalloc::kLog;
 using tcmalloc::kCrash;
 using tcmalloc::Log;
 using tcmalloc::PageHeap;
-using tcmalloc::PageHeapAllocator;
-using tcmalloc::SizeMap;
 using tcmalloc::Span;
 using tcmalloc::StackTrace;
 using tcmalloc::Static;
@@ -163,10 +154,6 @@ using tcmalloc::TestingPortal;
 DECLARE_double(tcmalloc_release_rate);
 DECLARE_int64(tcmalloc_heap_limit_mb);
 
-#ifndef NO_HEAP_CHECK
-DECLARE_string(heap_check);
-#endif
-
 // Those common architectures are known to be safe w.r.t. aliasing function
 // with "extra" unused args to function with fewer arguments (e.g.
 // tc_delete_nothrow being aliased to tc_delete).
@@ -189,96 +176,57 @@ DECLARE_string(heap_check);
 #define TC_ALIAS(name) __attribute__((alias(#name)))
 #endif
 
-// We already declared these functions in tcmalloc.h, but we have to
-// declare them again to give them an ATTRIBUTE_SECTION: we want to
-// put all callers of MallocHook::Invoke* in this module into
-// ATTRIBUTE_SECTION(google_malloc) section, so that
-// MallocHook::GetCallerStackTrace can function accurately.
-#ifndef _WIN32   // windows doesn't have attribute_section, so don't bother
 extern "C" {
-  void* tc_malloc(size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_free(void* ptr) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_free_sized(void* ptr, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_realloc(void* ptr, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_calloc(size_t nmemb, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_cfree(void* ptr) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_malloc(size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_free(void* ptr) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_free_sized(void* ptr, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_free_aligned_sized(void* ptr, size_t align, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_realloc(void* ptr, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_calloc(size_t nmemb, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_cfree(void* ptr) PERFTOOLS_NOTHROW;
 
-  void* tc_memalign(size_t __alignment, size_t __size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  int tc_posix_memalign(void** ptr, size_t align, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_valloc(size_t __size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_pvalloc(size_t __size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_memalign(size_t __alignment, size_t __size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE int tc_posix_memalign(void** ptr, size_t align, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_valloc(size_t __size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_pvalloc(size_t __size) PERFTOOLS_NOTHROW;
 
-  void tc_malloc_stats(void) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  int tc_mallopt(int cmd, int value) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-#ifdef HAVE_STRUCT_MALLINFO
-  struct mallinfo tc_mallinfo(void) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void tc_malloc_stats(void) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE int tc_mallopt(int cmd, int value) PERFTOOLS_NOTHROW;
+#if GPERFTOOLS_HAS_MALLINFO
+  ATTRIBUTE_NOINLINE struct mallinfo tc_mallinfo(void) PERFTOOLS_NOTHROW;
 #endif
-#ifdef HAVE_STRUCT_MALLINFO2
-  struct mallinfo2 tc_mallinfo2(void) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+#ifdef GPERFTOOLS_HAS_MALLINFO2
+  ATTRIBUTE_NOINLINE struct mallinfo2 tc_mallinfo2(void) PERFTOOLS_NOTHROW;
 #endif
 
-  void* tc_new(size_t size)
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_delete(void* p) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_delete_sized(void* p, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_newarray(size_t size)
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray(void* p) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray_sized(void* p, size_t size) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_new(size_t size);
+  ATTRIBUTE_NOINLINE void tc_delete(void* p) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_delete_sized(void* p, size_t size) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_newarray(size_t size);
+  ATTRIBUTE_NOINLINE void tc_deletearray(void* p) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_deletearray_sized(void* p, size_t size) PERFTOOLS_NOTHROW;
 
   // And the nothrow variants of these:
-  void* tc_new_nothrow(size_t size, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_newarray_nothrow(size_t size, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_new_nothrow(size_t size, const std::nothrow_t&) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_newarray_nothrow(size_t size, const std::nothrow_t&) PERFTOOLS_NOTHROW;
   // Surprisingly, standard C++ library implementations use a
   // nothrow-delete internally.  See, eg:
   // http://www.dinkumware.com/manuals/?manual=compleat&page=new.html
-  void tc_delete_nothrow(void* ptr, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray_nothrow(void* ptr, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void tc_delete_nothrow(void* ptr, const std::nothrow_t&) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_deletearray_nothrow(void* ptr, const std::nothrow_t&) PERFTOOLS_NOTHROW;
 
-  void* tc_new_aligned(size_t size, std::align_val_t al)
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_delete_aligned(void* p, std::align_val_t al) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_delete_sized_aligned(void* p, size_t size, std::align_val_t al) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_newarray_aligned(size_t size, std::align_val_t al)
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray_aligned(void* p, std::align_val_t al) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray_sized_aligned(void* p, size_t size, std::align_val_t al) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_new_aligned(size_t size, std::align_val_t al);
+  ATTRIBUTE_NOINLINE void tc_delete_aligned(void* p, std::align_val_t al) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_delete_sized_aligned(void* p, size_t size, std::align_val_t al) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_newarray_aligned(size_t size, std::align_val_t al);
+  ATTRIBUTE_NOINLINE void tc_deletearray_aligned(void* p, std::align_val_t al) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_deletearray_sized_aligned(void* p, size_t size, std::align_val_t al) PERFTOOLS_NOTHROW;
 
   // And the nothrow variants of these:
-  void* tc_new_aligned_nothrow(size_t size, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void* tc_newarray_aligned_nothrow(size_t size, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_delete_aligned_nothrow(void* ptr, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
-  void tc_deletearray_aligned_nothrow(void* ptr, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE void* tc_new_aligned_nothrow(size_t size, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void* tc_newarray_aligned_nothrow(size_t size, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_delete_aligned_nothrow(void* ptr, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW;
+  ATTRIBUTE_NOINLINE void tc_deletearray_aligned_nothrow(void* ptr, std::align_val_t al, const std::nothrow_t&) PERFTOOLS_NOTHROW;
 
   // Some non-standard extensions that we support.
 
@@ -286,10 +234,8 @@ extern "C" {
   //    OS X: malloc_size()
   //    glibc: malloc_usable_size()
   //    Windows: _msize()
-  size_t tc_malloc_size(void* p) PERFTOOLS_NOTHROW
-      ATTRIBUTE_SECTION(google_malloc);
+  ATTRIBUTE_NOINLINE size_t tc_malloc_size(void* p) PERFTOOLS_NOTHROW;
 }  // extern "C"
-#endif  // #ifndef _WIN32
 
 // ----------------------- IMPLEMENTATION -------------------------------
 
@@ -310,6 +256,9 @@ ATTRIBUTE_NOINLINE void InvalidFree(void* ptr) {
 }
 
 size_t InvalidGetAllocatedSize(const void* ptr) {
+  if (tcmalloc::IsEmergencyPtr(ptr)) {
+    return tcmalloc::EmergencyAllocatedSize(ptr);
+  }
   Log(kCrash, __FILE__, __LINE__,
       "Attempt to get the size of an invalid pointer", ptr);
   return 0;
@@ -326,10 +275,10 @@ struct TCMallocStats {
   PageHeap::Stats pageheap;   // Stats from page heap
 };
 
-// Get stats into "r".  Also, if class_count != NULL, class_count[k]
+// Get stats into "r".  Also, if class_count != nullptr, class_count[k]
 // will be set to the total number of objects of size class k in the
 // central cache, transfer cache, and per-thread caches. If small_spans
-// is non-NULL, it is filled.  Same for large_spans.
+// is non-nullptr, it is filled.  Same for large_spans.
 static void ExtractStats(TCMallocStats* r, uint64_t* class_count,
                          PageHeap::SmallSpanStats* small_spans,
                          PageHeap::LargeSpanStats* large_spans) {
@@ -358,10 +307,10 @@ static void ExtractStats(TCMallocStats* r, uint64_t* class_count,
     ThreadCache::GetThreadStats(&r->thread_bytes, class_count);
     r->metadata_bytes = tcmalloc::metadata_system_bytes();
     r->pageheap = Static::pageheap()->StatsLocked();
-    if (small_spans != NULL) {
+    if (small_spans != nullptr) {
       Static::pageheap()->GetSmallSpanStatsLocked(small_spans);
     }
-    if (large_spans != NULL) {
+    if (large_spans != nullptr) {
       Static::pageheap()->GetLargeSpanStatsLocked(large_spans);
     }
   }
@@ -380,7 +329,7 @@ static void DumpStats(TCMalloc_Printer* out, int level) {
   if (level >= 2) {
     ExtractStats(&stats, class_count, &small, &large);
   } else {
-    ExtractStats(&stats, NULL, NULL, NULL);
+    ExtractStats(&stats, nullptr, nullptr, nullptr);
   }
 
   static const double MiB = 1048576.0;
@@ -550,10 +499,7 @@ namespace tcmalloc {
 
 TestingPortal::~TestingPortal() = default;
 
-// implemented in heap-checker.cc
-extern void DoIterateMemoryRegionMap(tcmalloc::FunctionRef<void(const void*)> callback);
-
-class ATTRIBUTE_HIDDEN TestingPortalImpl : public TestingPortal {
+class ATTRIBUTE_VISIBILITY_HIDDEN TestingPortalImpl : public TestingPortal {
 public:
   ~TestingPortalImpl() override = default;
 
@@ -591,6 +537,10 @@ public:
     return kUseEmergencyMalloc;
   }
 
+  bool IsEmergencyPtr(void* ptr) override {
+    return tcmalloc::IsEmergencyPtr(ptr);
+  }
+
   void WithEmergencyMallocEnabled(FunctionRef<void()> body) override {
     auto body_adaptor = [body] (bool stacktrace_allowed) {
       CHECK(stacktrace_allowed);
@@ -600,19 +550,17 @@ public:
     ThreadCachePtr::WithStacktraceScope(ref.fn, ref.data);
   }
 
-  std::string_view GetHeapCheckFlag() override {
-#ifndef NO_HEAP_CHECK
-    return FLAGS_heap_check;
-#else
-    return "";
-#endif
-  }
-  void IterateMemoryRegionMap(FunctionRef<void(const void*)> callback) override {
-#ifndef NO_HEAP_CHECK
-    DoIterateMemoryRegionMap(callback);
-#endif
+  uint32_t GetSizeClass(void* p) override {
+    PageID pageid = reinterpret_cast<uintptr_t>(p) >> kPageShift;
+    Span* span = Static::pageheap()->GetDescriptor(pageid);
+    return span->sizeclass;
   }
 
+  void* RunReallocWithCallback(
+    void* old_ptr, size_t new_size,
+    void (*invalid_free_fn)(void*),
+    size_t (*invalid_get_size_fn)(const void*)) override;
+
   static TestingPortalImpl* Get() {
     static TestingPortalImpl* ptr = ([] () {
       static StaticStorage<TestingPortalImpl> storage;
@@ -708,11 +656,11 @@ class TCMallocImplementation : public MallocExtension {
   }
 
   virtual bool GetNumericProperty(const char* name, size_t* value) {
-    ASSERT(name != NULL);
+    ASSERT(name != nullptr);
 
     if (strcmp(name, "generic.current_allocated_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.pageheap.system_bytes
                - stats.thread_bytes
                - stats.central_bytes
@@ -724,14 +672,14 @@ class TCMallocImplementation : public MallocExtension {
 
     if (strcmp(name, "generic.heap_size") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.pageheap.system_bytes;
       return true;
     }
 
     if (strcmp(name, "generic.total_physical_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.pageheap.system_bytes + stats.metadata_bytes -
                stats.pageheap.unmapped_bytes;
       return true;
@@ -748,21 +696,21 @@ class TCMallocImplementation : public MallocExtension {
 
     if (strcmp(name, "tcmalloc.central_cache_free_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.central_bytes;
       return true;
     }
 
     if (strcmp(name, "tcmalloc.transfer_cache_free_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.transfer_bytes;
       return true;
     }
 
     if (strcmp(name, "tcmalloc.thread_cache_free_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.thread_bytes;
       return true;
     }
@@ -840,7 +788,7 @@ class TCMallocImplementation : public MallocExtension {
 
     if (strcmp(name, "tcmalloc.current_total_thread_cache_bytes") == 0) {
       TCMallocStats stats;
-      ExtractStats(&stats, NULL, NULL, NULL);
+      ExtractStats(&stats, nullptr, nullptr, nullptr);
       *value = stats.thread_bytes;
       return true;
     }
@@ -863,6 +811,11 @@ class TCMallocImplementation : public MallocExtension {
       return true;
     }
 
+    if (strcmp(name, "tcmalloc.sample_parameter") == 0) {
+      *value = FLAGS_tcmalloc_sample_parameter;
+      return true;
+    }
+
     if (TestingPortal** portal = TestingPortal::CheckGetPortal(name, value); portal) {
       *portal = TestingPortalImpl::Get();
       *value = 1;
@@ -873,7 +826,7 @@ class TCMallocImplementation : public MallocExtension {
   }
 
   virtual bool SetNumericProperty(const char* name, size_t value) {
-    ASSERT(name != NULL);
+    ASSERT(name != nullptr);
 
     if (strcmp(name, "tcmalloc.max_total_thread_cache_bytes") == 0) {
       SpinLockHolder l(Static::pageheap_lock());
@@ -898,6 +851,15 @@ class TCMallocImplementation : public MallocExtension {
       return true;
     }
 
+    if (strcmp(name, "tcmalloc.sample_parameter") == 0) {
+      FLAGS_tcmalloc_sample_parameter = value;
+      // By clearing current thread's cache we force next allocations
+      // to read freshly updated sample parameter. This is only going
+      // to affect current thread, but this is better than nothing.
+      MallocExtension::instance()->MarkThreadIdle();
+      return true;
+    }
+
     return false;
   }
 
@@ -945,7 +907,7 @@ class TCMallocImplementation : public MallocExtension {
     // num_bytes might be less than one page.  If we pass zero to
     // ReleaseAtLeastNPages, it won't do anything, so we release a whole
     // page now and let extra_bytes_released_ smooth it out over time.
-    Length num_pages = max<Length>(num_bytes >> kPageShift, 1);
+    Length num_pages = std::max<Length>(num_bytes >> kPageShift, 1);
     size_t bytes_released = Static::pageheap()->ReleaseAtLeastNPages(
         num_pages) << kPageShift;
     if (bytes_released > num_bytes) {
@@ -988,10 +950,13 @@ class TCMallocImplementation : public MallocExtension {
       return kOwned;
     }
     const Span *span = Static::pageheap()->GetDescriptor(p);
-    return span ? kOwned : kNotOwned;
+    if (span) {
+      return kOwned;
+    }
+    return tcmalloc::IsEmergencyPtr(ptr) ? kOwned : kNotOwned;
   }
 
-  virtual void GetFreeListSizes(vector<MallocExtension::FreeListInfo>* v) {
+  virtual void GetFreeListSizes(std::vector<MallocExtension::FreeListInfo>* v) {
     static const char kCentralCacheType[] = "tcmalloc.central";
     static const char kTransferCacheType[] = "tcmalloc.transfer";
     static const char kThreadCacheType[] = "tcmalloc.thread";
@@ -1057,7 +1022,7 @@ class TCMallocImplementation : public MallocExtension {
     // large spans: mapped
     MallocExtension::FreeListInfo span_info;
     span_info.type = kLargeSpanType;
-    span_info.max_object_size = (numeric_limits<size_t>::max)();
+    span_info.max_object_size = (std::numeric_limits<size_t>::max)();
     span_info.min_object_size = kMaxPages << kPageShift;
     span_info.total_bytes_free = large.normal_pages << kPageShift;
     v->push_back(span_info);
@@ -1118,7 +1083,7 @@ static bool size_class_with_alignment(size_t size, size_t align, uint32_t* cl) {
 // ThreadCache::InitModule is not inlined which would cause nallocx to
 // become non-leaf function with stack frame and stack spills.
 static ATTRIBUTE_NOINLINE size_t nallocx_slow(size_t size, int flags) {
-  if (PREDICT_FALSE(!Static::IsInited())) ThreadCache::InitModule();
+  ThreadCache::EnsureMallocInitialized();
 
   size_t align = static_cast<size_t>(1ull << (flags & 0x3f));
   uint32_t cl;
@@ -1191,12 +1156,12 @@ TCMallocGuard::TCMallocGuard() {
 
 TCMallocGuard::~TCMallocGuard() {
   if (--tcmallocguard_refcount == 0) {
-    const char* env = NULL;
+    const char* env = nullptr;
     if (!RunningOnValgrind()) {
       // Valgrind uses it's own malloc so we cannot do MALLOCSTATS
       env = getenv("MALLOCSTATS");
     }
-    if (env != NULL) {
+    if (env != nullptr) {
       int level = atoi(env);
       if (level < 1) level = 1;
       PrintStats(level);
@@ -1230,7 +1195,7 @@ static ATTRIBUTE_UNUSED bool CheckCachedSizeClass(void *ptr) {
 }
 
 static ALWAYS_INLINE void* CheckedMallocResult(void *result) {
-  ASSERT(result == NULL || CheckCachedSizeClass(result));
+  ASSERT(result == nullptr || CheckCachedSizeClass(result));
   return result;
 }
 
@@ -1249,8 +1214,8 @@ static void* DoSampledAllocation(size_t size) {
   // Allocate span
   auto pages = tcmalloc::pages(size == 0 ? 1 : size);
   Span *span = Static::pageheap()->New(pages);
-  if (PREDICT_FALSE(span == NULL)) {
-    return NULL;
+  if (PREDICT_FALSE(span == nullptr)) {
+    return nullptr;
   }
 
   SpinLockHolder h(Static::pageheap_lock());
@@ -1290,8 +1255,8 @@ void* handle_oom(malloc_fn retry_fn,
   errno = ENOMEM;
   if (!from_operator && !tc_new_mode) {
     // we're out of memory in C library function (malloc etc) and no
-    // "new mode" forced on us. Just return NULL
-    return NULL;
+    // "new mode" forced on us. Just return nullptr
+    return nullptr;
   }
 
   // we're OOM in operator new or "new mode" is set. We might have to
@@ -1335,7 +1300,7 @@ void* handle_oom(malloc_fn retry_fn,
     // we get here if new_handler returns successfully. So we retry
     // allocation.
     void* rv = retry_fn(retry_arg);
-    if (rv != NULL) {
+    if (rv != nullptr) {
       return rv;
     }
 
@@ -1445,7 +1410,7 @@ static void* do_malloc_pages(ThreadCache* heap, size_t size) {
     result = DoSampledAllocation(size);
   } else {
     Span* span = Static::pageheap()->New(num_pages);
-    result = (PREDICT_FALSE(span == NULL) ? NULL : SpanToMallocResult(span));
+    result = (PREDICT_FALSE(span == nullptr) ? nullptr : SpanToMallocResult(span));
   }
 
   if (should_report_large(num_pages)) {
@@ -1455,7 +1420,7 @@ static void* do_malloc_pages(ThreadCache* heap, size_t size) {
 }
 
 static void *nop_oom_handler(size_t size) {
-  return NULL;
+  return nullptr;
 }
 
 ALWAYS_INLINE void* do_malloc(size_t size) {
@@ -1491,7 +1456,7 @@ static void *retry_malloc(void* size) {
 
 ALWAYS_INLINE void* do_malloc_or_cpp_alloc(size_t size) {
   void *rv = do_malloc(size);
-  if (PREDICT_TRUE(rv != NULL)) {
+  if (PREDICT_TRUE(rv != nullptr)) {
     return rv;
   }
   return handle_oom(retry_malloc, reinterpret_cast<void *>(size),
@@ -1501,10 +1466,10 @@ ALWAYS_INLINE void* do_malloc_or_cpp_alloc(size_t size) {
 ALWAYS_INLINE void* do_calloc(size_t n, size_t elem_size) {
   // Overflow check
   const size_t size = n * elem_size;
-  if (elem_size != 0 && size / elem_size != n) return NULL;
+  if (elem_size != 0 && size / elem_size != n) return nullptr;
 
   void* result = do_malloc_or_cpp_alloc(size);
-  if (result != NULL) {
+  if (result != nullptr) {
     size_t total_size = size;
     if (!tcmalloc::IsEmergencyPtr(result)) {
       // On windows we support recalloc (which was apparently
@@ -1522,9 +1487,9 @@ ALWAYS_INLINE void* do_calloc(size_t n, size_t elem_size) {
   return result;
 }
 
-// If ptr is NULL, do nothing.  Otherwise invoke the given function.
+// If ptr is nullptr, do nothing.  Otherwise invoke the given function.
 inline void free_null_or_invalid(void* ptr, void (*invalid_free_fn)(void*)) {
-  if (ptr != NULL) {
+  if (ptr != nullptr) {
     (*invalid_free_fn)(ptr);
   }
 }
@@ -1542,7 +1507,7 @@ static ATTRIBUTE_NOINLINE void do_free_pages(Span* span, void* ptr) {
       StackTrace* st = reinterpret_cast<StackTrace*>(span->objects);
       tcmalloc::DLL_Remove(span);
       Static::stacktrace_allocator()->Delete(st);
-      span->objects = NULL;
+      span->objects = nullptr;
     }
   });
 }
@@ -1586,7 +1551,7 @@ void do_free_with_callback(void* ptr,
     if (PREDICT_FALSE(!cache_hit)) {
       Span* span  = Static::pageheap()->GetDescriptor(p);
       if (PREDICT_FALSE(!span)) {
-        // span can be NULL because the pointer passed in is NULL or invalid
+        // span can be nullptr because the pointer passed in is nullptr or invalid
         // (not something returned by malloc or friends), or because the
         // pointer was allocated with some other allocator besides
         // tcmalloc.  The latter can happen if tcmalloc is linked in via
@@ -1599,7 +1564,7 @@ void do_free_with_callback(void* ptr,
       cl = span->sizeclass;
       if (PREDICT_FALSE(cl == 0)) {
         ASSERT(reinterpret_cast<uintptr_t>(ptr) % kPageSize == 0);
-        ASSERT(span != NULL && span->start == p);
+        ASSERT(span != nullptr && span->start == p);
         do_free_pages(span, ptr);
         return;
       }
@@ -1609,7 +1574,7 @@ void do_free_with_callback(void* ptr,
     }
   }
 
-  if (PREDICT_TRUE(heap != NULL)) {
+  if (PREDICT_TRUE(heap != nullptr)) {
     ASSERT(Static::IsInited());
     // If we've hit initialized thread cache, so we're done.
     heap->Deallocate(ptr, cl);
@@ -1628,7 +1593,7 @@ void do_free_with_callback(void* ptr,
   }
 
   // Otherwise, delete directly into central cache
-  tcmalloc::SLL_SetNext(ptr, NULL);
+  tcmalloc::SLL_SetNext(ptr, nullptr);
   Static::central_cache()[cl].InsertRange(ptr, ptr, 1);
 }
 
@@ -1641,7 +1606,7 @@ ALWAYS_INLINE void do_free(void* ptr) {
 // speed.  If you change this function, look at that one too.
 inline size_t GetSizeWithCallback(const void* ptr,
                                   size_t (*invalid_getsize_fn)(const void*)) {
-  if (ptr == NULL)
+  if (ptr == nullptr)
     return 0;
   const PageID p = reinterpret_cast<uintptr_t>(ptr) >> kPageShift;
   uint32_t cl;
@@ -1650,7 +1615,7 @@ inline size_t GetSizeWithCallback(const void* ptr,
   }
 
   const Span *span = Static::pageheap()->GetDescriptor(p);
-  if (PREDICT_FALSE(span == NULL)) {  // means we do not own this memory
+  if (PREDICT_FALSE(span == nullptr)) {  // means we do not own this memory
     return (*invalid_getsize_fn)(ptr);
   }
 
@@ -1672,56 +1637,59 @@ ALWAYS_INLINE void* do_realloc_with_callback(
     void* old_ptr, size_t new_size,
     void (*invalid_free_fn)(void*),
     size_t (*invalid_get_size_fn)(const void*)) {
-  // Get the size of the old entry
-  const size_t old_size = GetSizeWithCallback(old_ptr, invalid_get_size_fn);
 
-  // Reallocate if the new size is larger than the old size,
-  // or if the new size is significantly smaller than the old size.
-  // We do hysteresis to avoid resizing ping-pongs:
-  //    . If we need to grow, grow to max(new_size, old_size * 1.X)
-  //    . Don't shrink unless new_size < old_size * 0.Y
-  // X and Y trade-off time for wasted space.  For now we do 1.25 and 0.5.
-  const size_t min_growth = min(old_size / 4,
-      (std::numeric_limits<size_t>::max)() - old_size);  // Avoid overflow.
-  const size_t lower_bound_to_grow = old_size + min_growth;
-  const size_t upper_bound_to_shrink = old_size / 2ul;
-  if ((new_size > old_size) || (new_size < upper_bound_to_shrink)) {
-    // Need to reallocate.
-    void* new_ptr = NULL;
+  size_t currently_usable = GetSizeWithCallback(
+    old_ptr,
+    +[] (const void* invalid_ptr) -> size_t {
+      // If we're passed "invalid" object, have free step handle
+      // either falling back to "pre-patch" msvc runtime, or reporting
+      // invalid free. Returning 0 makes us bypass "keep in place"
+      // option.
+      return 0;
+    });
 
-    if (new_size > old_size && new_size < lower_bound_to_grow) {
-      new_ptr = do_malloc_or_cpp_alloc(lower_bound_to_grow);
-    }
-    if (new_ptr == NULL) {
-      // Either new_size is not a tiny increment, or last do_malloc failed.
-      new_ptr = do_malloc_or_cpp_alloc(new_size);
-    }
-    if (PREDICT_FALSE(new_ptr == NULL)) {
-      return NULL;
-    }
-    MallocHook::InvokeNewHook(new_ptr, new_size);
-    memcpy(new_ptr, old_ptr, ((old_size < new_size) ? old_size : new_size));
-    MallocHook::InvokeDeleteHook(old_ptr);
-    // We could use a variant of do_free() that leverages the fact
-    // that we already know the sizeclass of old_ptr.  The benefit
-    // would be small, so don't bother.
-    do_free_with_callback(old_ptr, invalid_free_fn, false, 0);
-    return new_ptr;
-  } else {
-    // We still need to call hooks to report the updated size:
-    MallocHook::InvokeDeleteHook(old_ptr);
-    MallocHook::InvokeNewHook(old_ptr, new_size);
+  if (tc_nallocx(new_size, 0) == currently_usable) {
+    // stay in place. Communicate new logical size to the hooks.
+    //
+    // Note: we're skipping sampling updates because we consider this
+    // "allocation" 0-cost.
+    tcmalloc::InvokeDeleteHook(old_ptr);
+    tcmalloc::InvokeNewHook(old_ptr, new_size);
     return old_ptr;
   }
+
+  void* new_ptr = do_malloc_or_cpp_alloc(new_size);
+  if (new_ptr == nullptr) {
+    // NOTE: Setting ENOMEM or any other kind of OOM handling has been
+    // done in the do_malloc_or_cpp_alloc thingy.
+    return nullptr;
+  }
+
+  if (currently_usable == 0) {
+    // We had the "not our chunk of memory" case when fetching
+    // currently_usable size. So lets get real old size so that memcpy
+    // below is handled correctly.
+    currently_usable = invalid_get_size_fn(old_ptr);
+  }
+
+  tcmalloc::InvokeNewHook(new_ptr, new_size);
+  memcpy(new_ptr, old_ptr, std::min(currently_usable, new_size));
+  tcmalloc::InvokeDeleteHook(old_ptr);
+
+  // We could use a variant of do_free() that leverages the fact that
+  // we already know the sizeclass of old_ptr.  The benefit would be
+  // small, so don't bother.
+  do_free_with_callback(old_ptr, invalid_free_fn, false, 0);
+  return new_ptr;
 }
 
 static ALWAYS_INLINE
 void* do_memalign_pages(size_t align, size_t size) {
   ASSERT((align & (align - 1)) == 0);
   ASSERT(align > kPageSize);
-  if (size + align < size) return NULL;         // Overflow
+  if (size + align < size) return nullptr;         // Overflow
 
-  if (PREDICT_FALSE(Static::pageheap() == NULL)) ThreadCache::InitModule();
+  if (PREDICT_FALSE(Static::pageheap() == nullptr)) ThreadCache::InitModule();
 
   // Allocate at least one byte to avoid boundary conditions below
   if (size == 0) size = 1;
@@ -1747,11 +1715,11 @@ inline int do_mallopt(int cmd, int value) {
   return 1;     // Indicates error
 }
 
-#if defined(HAVE_STRUCT_MALLINFO) || defined(HAVE_STRUCT_MALLINFO2)
+#if GPERFTOOLS_HAS_MALLINFO2 || GPERFTOOLS_HAS_MALLINFO
 template <typename Mallinfo>
 inline Mallinfo do_mallinfo() {
   TCMallocStats stats;
-  ExtractStats(&stats, NULL, NULL, NULL);
+  ExtractStats(&stats, nullptr, nullptr, nullptr);
 
   // Just some of the fields are filled in.
   Mallinfo info;
@@ -1777,13 +1745,13 @@ inline Mallinfo do_mallinfo() {
 
   return info;
 }
-#endif  // HAVE_STRUCT_MALLINFO || HAVE_STRUCT_MALLINFO2
+#endif  // GPERFTOOLS_HAS_MALLINFO{,2}
 
 }  // end unnamed namespace
 
 // As promised, the definition of this function, declared above.
 size_t TCMallocImplementation::GetAllocatedSize(const void* ptr) {
-  if (ptr == NULL)
+  if (ptr == nullptr)
     return 0;
   ASSERT(TCMallocImplementation::GetOwnership(ptr)
          != TCMallocImplementation::kNotOwned);
@@ -1823,6 +1791,16 @@ extern "C" PERFTOOLS_DLL_DECL int tc_query_new_mode() PERFTOOLS_NOTHROW {
   return tc_new_mode;
 }
 
+namespace tcmalloc {
+void* TestingPortalImpl::RunReallocWithCallback(
+    void* old_ptr, size_t new_size,
+    void (*invalid_free_fn)(void*),
+    size_t (*invalid_get_size_fn)(const void*)) {
+  return do_realloc_with_callback(old_ptr, new_size, invalid_free_fn, invalid_get_size_fn);
+}
+}
+
+
 #ifndef TCMALLOC_USING_DEBUGALLOCATION  // debugallocation.cc defines its own
 
 // CAVEAT: The code structure below ensures that MallocHook methods are always
@@ -1833,25 +1811,25 @@ extern "C" PERFTOOLS_DLL_DECL int tc_query_new_mode() PERFTOOLS_NOTHROW {
 namespace tcmalloc {
 
 
-static ATTRIBUTE_SECTION(google_malloc)
+static ATTRIBUTE_NOINLINE
 void invoke_hooks_and_free(void *ptr) {
-  MallocHook::InvokeDeleteHook(ptr);
+  tcmalloc::InvokeDeleteHook(ptr);
   do_free(ptr);
 }
 
-ATTRIBUTE_SECTION(google_malloc)
+ATTRIBUTE_NOINLINE
 void* cpp_throw_oom(size_t size) {
   return handle_oom(retry_malloc, reinterpret_cast<void *>(size),
                     true, false);
 }
 
-ATTRIBUTE_SECTION(google_malloc)
+ATTRIBUTE_NOINLINE
 void* cpp_nothrow_oom(size_t size) {
   return handle_oom(retry_malloc, reinterpret_cast<void *>(size),
                     true, true);
 }
 
-ATTRIBUTE_SECTION(google_malloc)
+ATTRIBUTE_NOINLINE
 void* malloc_oom(size_t size) {
   return handle_oom(retry_malloc, reinterpret_cast<void *>(size),
                     false, true);
@@ -1884,15 +1862,15 @@ template <void* OOMHandler(size_t)>
 ALWAYS_INLINE
 static void* do_allocate_full(size_t size) {
   void* p = do_malloc(size);
-  if (PREDICT_FALSE(p == NULL)) {
+  if (PREDICT_FALSE(p == nullptr)) {
     p = OOMHandler(size);
   }
-  MallocHook::InvokeNewHook(p, size);
+  tcmalloc::InvokeNewHook(p, size);
   return CheckedMallocResult(p);
 }
 
 #define AF(oom) \
-  ATTRIBUTE_SECTION(google_malloc)   \
+  ATTRIBUTE_NOINLINE   \
   void* allocate_full_##oom(size_t size) {   \
     return do_allocate_full<oom>(size);     \
   }
@@ -1925,18 +1903,18 @@ static void *retry_do_memalign(void *arg) {
   return do_memalign_pages(data->align, data->size);
 }
 
-static ATTRIBUTE_SECTION(google_malloc)
+static ATTRIBUTE_NOINLINE
 void* memalign_pages(size_t align, size_t size,
                      bool from_operator, bool nothrow) {
   void *rv = do_memalign_pages(align, size);
-  if (PREDICT_FALSE(rv == NULL)) {
+  if (PREDICT_FALSE(rv == nullptr)) {
     retry_memalign_data data;
     data.align = align;
     data.size = size;
     rv = handle_oom(retry_do_memalign, &data,
                     from_operator, nothrow);
   }
-  MallocHook::InvokeNewHook(rv, size);
+  tcmalloc::InvokeNewHook(rv, size);
   return CheckedMallocResult(rv);
 }
 
@@ -1965,7 +1943,7 @@ static void * malloc_fast_path(size_t size) {
 
   ThreadCache *cache = ThreadCachePtr::GetIfPresent();
 
-  if (PREDICT_FALSE(cache == NULL)) {
+  if (PREDICT_FALSE(cache == nullptr)) {
     return tcmalloc::dispatch_allocate_full<OOMHandler>(size);
   }
 
@@ -2065,7 +2043,7 @@ extern "C" PERFTOOLS_DLL_DECL void tc_deletearray_sized(void *p, size_t size) PE
 extern "C" PERFTOOLS_DLL_DECL void* tc_calloc(size_t n,
                                               size_t elem_size) PERFTOOLS_NOTHROW {
   void* result = do_calloc(n, elem_size);
-  MallocHook::InvokeNewHook(result, n * elem_size);
+  tcmalloc::InvokeNewHook(result, n * elem_size);
   return result;
 }
 
@@ -2080,15 +2058,15 @@ TC_ALIAS(tc_free);
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_realloc(void* old_ptr,
                                                size_t new_size) PERFTOOLS_NOTHROW {
-  if (old_ptr == NULL) {
+  if (old_ptr == nullptr) {
     void* result = do_malloc_or_cpp_alloc(new_size);
-    MallocHook::InvokeNewHook(result, new_size);
+    tcmalloc::InvokeNewHook(result, new_size);
     return result;
   }
   if (new_size == 0) {
-    MallocHook::InvokeDeleteHook(old_ptr);
+    tcmalloc::InvokeDeleteHook(old_ptr);
     do_free(old_ptr);
-    return NULL;
+    return nullptr;
   }
   if (PREDICT_FALSE(tcmalloc::IsEmergencyPtr(old_ptr))) {
     return tcmalloc::EmergencyRealloc(old_ptr, new_size);
@@ -2185,7 +2163,7 @@ extern "C" PERFTOOLS_DLL_DECL int tc_posix_memalign(
   }
 
   void* result = tc_memalign(align, size);
-  if (PREDICT_FALSE(result == NULL)) {
+  if (PREDICT_FALSE(result == nullptr)) {
     return ENOMEM;
   } else {
     *result_ptr = result;
@@ -2218,6 +2196,11 @@ extern "C" PERFTOOLS_DLL_DECL void tc_delete_aligned_nothrow(void* p, std::align
   free_fast_path(p);
 }
 
+extern "C" PERFTOOLS_DLL_DECL
+void tc_free_aligned_sized(void *ptr, size_t, size_t) PERFTOOLS_NOTHROW {
+  free_fast_path(ptr);
+}
+
 extern "C" PERFTOOLS_DLL_DECL void* tc_newarray_aligned(size_t size, std::align_val_t align)
 #ifdef TC_ALIAS
 TC_ALIAS(tc_new_aligned);
@@ -2291,13 +2274,13 @@ extern "C" PERFTOOLS_DLL_DECL int tc_mallopt(int cmd, int value) PERFTOOLS_NOTHR
   return do_mallopt(cmd, value);
 }
 
-#ifdef HAVE_STRUCT_MALLINFO
+#if GPERFTOOLS_HAS_MALLINFO
 extern "C" PERFTOOLS_DLL_DECL struct mallinfo tc_mallinfo(void) PERFTOOLS_NOTHROW {
   return do_mallinfo<struct mallinfo>();
 }
 #endif
 
-#ifdef HAVE_STRUCT_MALLINFO2
+#if GPERFTOOLS_HAS_MALLINFO2
 extern "C" PERFTOOLS_DLL_DECL struct mallinfo2 tc_mallinfo2(void) PERFTOOLS_NOTHROW {
   return do_mallinfo<struct mallinfo2>();
 }
@@ -2309,7 +2292,7 @@ extern "C" PERFTOOLS_DLL_DECL size_t tc_malloc_size(void* ptr) PERFTOOLS_NOTHROW
 
 extern "C" PERFTOOLS_DLL_DECL void* tc_malloc_skip_new_handler(size_t size)  PERFTOOLS_NOTHROW {
   void* result = do_malloc(size);
-  MallocHook::InvokeNewHook(result, size);
+  tcmalloc::InvokeNewHook(result, size);
   return result;
 }
 
